#!/usr/bin/perl
##
## fetch_stat.pl - v1.0 - 2006-06-22
##
## Fetches and summarizes stats from Dell PowerConnect 27xx
## Written by Joakim Andersson, joakim at iqcc dot se
##
## 1) Install the Perl-modules you might be missing
## 2) Adjust the settings below
## 3) Run the script. Will not run in background
##
## Arguments: 
## clear_stat	 - Clear the portstats when script starts
## clear_session - Forces a cached session to be ignored, useful for testing only 
##
## Version 2006-06-22
##   Initial public release
##

use strict;
use Digest::MD5  qw(md5_hex);
use LWP::UserAgent;
use Net::HTTP;
use Time::HiRes;

####################################################
##
## Settings
##

my $SWITCH = '192.168.0.12';		# IP of switch
my $USERNAME = 'admin';				# Username
my $PASSWORD = 'admin';				# Password

# File to store cookie, prevents logging in every time script is started
# Too many sessions will hang the web interface otherwise
# Leave empty to disable caching
my $COOKIE_FILE = 'dell_cookie.txt';

# File to write HTML-page to
# Leave empty to disable generation
my $HTML_FILE = 'dell.html';

# Poll ports this often, in seconds
my $refresh_rate = 15;

# Summarize these ports, and show these names
my %do_ports = (
	2 => 'Joxx',
	4 => 'Gandalf',
	5 => 'DM500+Xbox',
	6 => 'WLAN+Xbox',
	8 => 'Shadow',
	10 => 'Brother MFC',
	12 => 'DM7000',
);

# End of settings
##################################################

my @do_delta = qw(goodOctetRCV rxGood goodOctetSND txGood rx64Octets rx65TO127Octets rx128TO255Octets rx256TO511Octets rx512TO1023Octets rx1024ToMa );

my $TR_COLOR = '#FFFF80';	# #FE8080 <font color="#FFFF80"></font>

my $clear_session = 0;
my $clear_stat = 0;
my $cookie = '';
my %system = ();

# Looking for valid args
foreach (@ARGV)
{
	if ($_ eq 'clear_stat')
	{
		$clear_stat = 1;
	}
	elsif ($_ eq 'clear_session')
	{
		$clear_session = 1;
	}
}

# Reads cached cookie, if wanted
if ($COOKIE_FILE)
{
	open (COOKIE, $COOKIE_FILE) || warn "Unable to open cookie-file $COOKIE_FILE, first run ever?\n";
	$cookie = <COOKIE>;
	close COOKIE;
}

if ($clear_session && $cookie)
{
	$cookie = '';
	print "Cleared old session\n";
}
elsif ($cookie)
{
	$cookie =~ tr/\n\r\t//d;
	print "Trying to reuse old session '$cookie'\n";
}

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");
#$ua->default_header( 'Cookie' => $cookie ) if ($cookie);

my %STATS = ();			# To maintain delta-stats 
my %OLD_STATS = ();		# To cache previous values
my @do_ports = sort { $a <=> $b } keys %do_ports;

##
## Finds longest portname + clears stats if wanted
##

my $maxlen_portname = 10;
foreach my $portnum (@do_ports)
{
	my $this_len = length($do_ports{$portnum});
	$maxlen_portname = $this_len if ($maxlen_portname < $this_len);
	
	##
	## Clears stats of these ports if program called with arg clear_stat
	##
	
	if ($clear_stat)
	{
		print "Clearing stats for port $portnum...\n";
		&clear_stat($portnum);
	}
}

##
## Loops forever, hopefully
##

my $loop = 0;
my $runtime = 0;

while (1)
{
	$loop++;
	&login unless ($cookie);	# Logins when needed
	
	# Remembers the time when this loop started
	my $start_time = [ Time::HiRes::gettimeofday() ];
	
	##
	## Polling every defined port
	##
	
	foreach my $portnum (@do_ports)
	{
		# Initiates the stat-hash
		my %stat = ( 'time' => [ Time::HiRes::gettimeofday() ] );
		
		print "Polling port $portnum...\n";
		
		my $res = $ua->get("http://$SWITCH/portStats.htm?PortNo=$portnum",
		       'Cookie' => $cookie
			 );
		
		die "Unable to fetch /portStats.htm?PortNo=$portnum - ", $res->status_line, "\n" unless ($res->is_success);
		
		# Parsing content
		my $data = $res->content;
		$data =~ tr/\r//d;
		
		if ($data =~ /^<title>Login<\/title>$/m)
		{
			##
			## Invalid session!
			##
			
			print "Session invalid\n";
			
			$cookie = '';
			last;
		}
		
		##
		## Parses the data, is stored in javascript-variables in the beginning of the page
		##
		
		my $row = 0;
		foreach (split(/\n+/, $data))
		{
			$row++;
			#print "RAD $row: $_\n";
			
			last if (/^<\/script>$/);	# May need adjustment for future firmwares
			next unless (/^var (.+)="(\d*)";$/);
			
			my $name = $1;
			my $value = reverse $2;		# Values are stored reversed
			$value += 0;				# Perhaps unneeded, but zero is better that nothing
			
			next if ($stat{$name});		# Skips already found variables, a bug in the current firmware
			
			#print "  $2 -> $value\n";
			$stat{$name} = $value;
		}
		
		unless ($stat{'portIndex'})
		{
			print "BUG? We are logged in, but didn't obtain variable portIndex!\n";
			exit;
			#$cookie = '';
			#last;
		}
		
		##
		## Do we have old info, so we can compute delta-values?
		##
		
		if (defined $STATS{$portnum})
		{
			##
			## Yes we do, let's compute
			##
			
			my $old_stat = $STATS{$portnum};
			$stat{'link'} = $old_stat->{'link'};	# We doesn't fetch link-info every loop
			
			# Elapsed time since last run
			my $delta_time = $stat{'delta_time'} = Time::HiRes::tv_interval($old_stat->{'time'}, $stat{'time'});
			
			if ($delta_time > 0)
			{
				foreach (@do_delta)
				{
					$stat{"delta_$_"} = ($stat{$_} - $old_stat->{$_}) / $delta_time;
				}
			}
		}
		
		# Memorizes the previous stat for this port
		$OLD_STATS{$portnum} = $STATS{$portnum} if ($STATS{$portnum});
		$STATS{$portnum} = \%stat;
	}
	
	# Ends loop here if we don't have a valid session
	next unless ($cookie);
	
	##
	## We doesn't fetches portinfo every loop, since it takes a while
	## Currently fetches every 8th loop
	##
	
	if ($loop == 1 || ($loop % 8 == 0))
	{
		print "Polling link info...\n";
		
		my $res = $ua->get("http://$SWITCH/portConfig.htm",
		       'Cookie' => $cookie
			 );
		
		die "Unable to fetch /portConfig.htm - ", $res->status_line, "\n" unless ($res->is_success);
		
		# Content
		my $data = $res->content;
		$data =~ tr/\r//d;
		
		if ($data =~ /^<title>Login<\/title>$/m)
		{
			##
			## Invalid session!
			##
			
			print "Invalid session\n";
			
			$cookie = '';
		}
		else
		{
			##
			## Finds the info we needs
			##
			
			my $row = 0;
			my %vars = ();
			foreach (split(/\n+/, $data))
			{
				$row++;
				#print "RAD $row: $_\n";
				
				last if (/^<\/script>$/);	# May need adjustment in future firmwares
				next unless (/^var (.+)="(\d*)";$/);
				
				my $name = $1;
				my $value = $2;
				
				#next if ($stat{$name});
				#print "  $name -> $value\n";
				$vars{$name} = $value;
			}
			
			##
			## Parses info about every port
			##
			
			foreach my $port (1 .. $vars{'numberOfPorts'})
			{
				my $stat = $STATS{$port};
				
				# var portLinkList="0101110101011100000000";
				# var portSpeedList="0202110201012200333333";
				# var portDuplexList="0101110101011100222222";
				# var portANList="1111111111111111222222";
				# var portBPList="0000000000000000222222";
				# var portFcList="0100000100000000333333";

				my $index = $port - 1;
				my $portLink   = substr($vars{'portLinkList'},   $index, 1);
				my $portSpeed  = substr($vars{'portSpeedList'},  $index, 1);
				my $portDuplex = substr($vars{'portDuplexList'}, $index, 1);
				#my $portAN     = substr($vars{'portANList'},     $index, 1);
				my $portFc     = substr($vars{'portFcList'},     $index, 1);
				my $portBP     = substr($vars{'portBPList'},     $index, 1);
				
				my $link = ('10','100','1000')[$portSpeed];
				$link .= ('/Half','/Full')[$portDuplex];
				$link .= '/FC' if ($portFc);
				$link .= '/BP' if ($portBP);
				$link = '-' unless ($portLink);
				
				$stat->{'link'} = $link;
				
				#print "$port: $link\n";
			}
			
			# Also fetches system-info now
			&get_system_info;
		}
	}
	
	# Ends loop here if we don't have a valid session
	next unless ($cookie);
	
	# Ends loop here if we don't have a valid session
	next unless ($cookie);
	
	##
	## Show some info
	##
	
	if (1)
	{
		foreach my $portnum (@do_ports)
		{
			my $stat = $STATS{$portnum};
			next unless ($stat->{'delta_time'});
			
			my $name = $do_ports{$portnum};
			print $name, ' ' x ($maxlen_portname - length($name)), " - ";
			
			foreach (qw(goodOctetRCV rxGood goodOctetSND txGood))
			{
				my $value = $stat->{"delta_$_"};
				$value *= 8 if (/Octet/);
				
				$value = &shorten_num($value);
				$value .= (/Octet/) ? 'bps' : 'pps';
				
				print "    $_:", ' ' x (8 - length($value)), $value;
			}
			
			my $show = &shorten_num($stat->{'delta_time'}, 6);
			print "    time: ${show}s";
			print "\n";
		}
	}
	
	##
	## Runtime for this loop
	##
	
	$runtime = Time::HiRes::tv_interval($start_time, [ Time::HiRes::gettimeofday() ]);
	my $sleep = $refresh_rate - $runtime;
	$sleep = 5 if ($sleep < 1);
	
	##
	## Generates web-page if wanted
	##
	
	&generate_html if ($HTML_FILE);
	
	##
	## Pauses
	##
	
	print "Loop took $runtime secs, sleeps $sleep secs...\n";
	Time::HiRes::sleep($sleep);
}

exit;


##
## Create the HTML-page
##

sub generate_html
{
	#print "HTML refresh: $refresh\n";
	
	open (HTML, ">$HTML_FILE") || warn "Unable to write to html-file '$HTML_FILE'!";
	print HTML <<END;
<html>
<head>
<title>Statistics for $SWITCH - $system{'systemName'} $system{'locationName'}</title>
<meta http-equiv="Refresh" content="$refresh_rate">
</head>

<body bgcolor="White">

<center>
<p>
<table border="0" cellpadding="4" cellspacing="0" style="font-family: Verdana, Geneva, Arial, Helvetica, sans-serif; font-size: 10;" width="980">

<tr align=left valign=bottom>
<th align=right rowspan=2 bgcolor="silver">#</th>
<th rowspan=2 bgcolor="silver">Equipment</th>
<th colspan=6 bgcolor="lime">Switch -&gt; Equipment:</th>
<th colspan=2 bgcolor="red">Errors</th>
<td bgcolor="silver"></td>
<th colspan=6 bgcolor="lime">Equipment -&gt; Switch:</th>
<th colspan=5 bgcolor="red">Errors</th>
</tr>
<tr align=left valign=bottom>
<th bgcolor="lime" colspan=2>Speed</th>
<th bgcolor="lime" colspan=2>TOT</th>
<th bgcolor="lime">Bcast</th>
<th bgcolor="lime">Mcast</th>
<th bgcolor="red">Drop</th>
<th bgcolor="red">Coll</th>
<td bgcolor="silver"></td>

<th bgcolor="lime" colspan=2>Speed</th>
<th bgcolor="lime" colspan=2>TOT</th>
<th bgcolor="lime">Bcast</th>
<th bgcolor="lime">Mcast</th>
<th bgcolor="red">FCS</th>
<th bgcolor="red">Usize</th>
<th bgcolor="red">Osize</th>
<th bgcolor="red">Frag</th>
<th bgcolor="red">Jabb</th>
</tr>
END
	#<th bgcolor="red">TOT</th>
		
	foreach my $portnum (@do_ports)
	{
		##
		## Varje port
		##
		
		my $stat = $STATS{$portnum};
		my $old_stat = $OLD_STATS{$portnum};
		my $is_err = 0;
		
		print HTML <<END;
<tr align=right valign=top onmouseover="this.style.backgroundColor='$TR_COLOR'" onmouseout="this.style.backgroundColor=''">
<th bgcolor="silver">$portnum</th>
<td align=left>$do_ports{$portnum}</td>
END
			
		foreach (qw(
delta_goodOctetSND delta_txGood goodOctetSND txGood txBroadcast txMulticast
is_err txDrop txCollisions no_err
space
delta_goodOctetRCV delta_rxGood goodOctetRCV rxGood rxBroadcast rxMulticast
is_err rxFCS rxUnderSize rxOverSize rxFragment rxJabber no_err
			))	# txError rxError 
		{
			##
			## Visar vrden
			##
			
			if ($_ eq 'space')
			{
				print HTML "<td></td>";
				next;
			}
			elsif ($_ eq 'is_err')
			{
				$is_err = 1;
				next;
			}
			elsif ($_ eq 'no_err')
			{
				$is_err = 0;
				next;
			}
			
			my $value = $stat->{$_};
			
			my $make_boldred = ($is_err && defined $old_stat->{$_} && ($value ne $old_stat->{$_}));
			
			if (/^delta_/)
			{
				# Delta-variable
				if ($value > 0)
				{
					$value *= 8 if (/Octet/);	# We want bit/s not byte/s
					my $tmp = $value;
					$value = &shorten_num($value);
					$value .= (/Octet[^s]/) ? 'bps' : 'pps';
					
					$value = "<font color=\"silver\">$value</font>" if ($tmp < 5 && $value =~ /pps$/);
					$value = "<font color=\"silver\">$value</font>" if ($tmp < 1024 && $value =~ /bps$/);

				}
				else
				{
					$value = '';
				}
			}
			elsif (/^(goodOctetRCV|txGood|goodOctetSND|rxGood)$/)
			{
				if ($value > 0)
				{
					$value = &shorten_num($value);
					$value .= (/Octet/) ? 'B' : 'p';
				}
				else
				{
					$value = '';
				}
			}
			else
			{
				$value = int($value);

				if ($value eq '0')
				{
					$value = "<font color=\"silver\">$value</font>";
				}
			}
			
			$value = "<b><font color=red>$value</font></b>" if ($make_boldred);
			$value = "<b>$value</b>" if ($value =~ /(kpps|Mbps)$/);
			print HTML "<td>$value</td>";
		}
		
		print HTML "</tr>\n";
	}
	
	print HTML <<END;
</table>

<br><br><br>
<table border="0" cellpadding="4" cellspacing="0" style="font-family: Verdana, Geneva, Arial, Helvetica, sans-serif; font-size: 10;" width=980>

<tr align=left valign=bottom>
<th align=right rowspan=2 bgcolor="silver">#</th>
<th rowspan=2 bgcolor="silver">Equipment</th>
<th bgcolor="lime"></th>
<th bgcolor="silver" width=0></th>
<th colspan=23 bgcolor="lime">Equipment -> Switch: Amount of packets per framesize</th>
</tr>
<tr align=center valign=bottom>
<th bgcolor="lime">Link</th>
<th bgcolor="silver" width=0></th>
<th bgcolor="lime" colspan=3>64 byte</th>
<td bgcolor="silver" width=0></td>
<th bgcolor="lime" colspan=3>65-127 byte</th>
<td bgcolor="silver" width=0></td>
<th bgcolor="lime" colspan=3>128-255 byte</th>
<td bgcolor="silver" width=0></td>
<th bgcolor="lime" colspan=3>256-511 byte</th>
<td bgcolor="silver" width=0></td>
<th bgcolor="lime" colspan=3>512-1024 byte</th>
<td bgcolor="silver" width=0></td>
<th bgcolor="lime" colspan=3>1025- byte</th>
</tr>
END
	
	foreach my $portnum (@do_ports)
	{
		##
		## Varje port
		##
		
		my $stat = $STATS{$portnum};
		my $old_stat = $OLD_STATS{$portnum};
		
		print HTML <<END;
<tr align=right valign=top onmouseover="this.style.backgroundColor='$TR_COLOR'" onmouseout="this.style.backgroundColor=''">
<th bgcolor="silver">$portnum</th>
<td align=left>$do_ports{$portnum}</td>
<td align=right>$stat->{'link'}</td>
<td></td>
END
		
		foreach (qw(
rx64Octets
rx65TO127Octets
rx128TO255Octets
rx256TO511Octets
rx512TO1023Octets
rx1024ToMa
			))
		{
			##
			## Vanliga vrdet
			##
			
			my $value = &shorten_num($stat->{$_}, 4);
			my $proc = 0;
			eval { $proc = int($stat->{$_} / $stat->{'rxGood'} * 100) };
			
			if ($value eq '0' || $proc < 10)
			{
				$value = "<font color=\"silver\">$value</font>";
			}
			
			print HTML "<td>$value</td>";
			
			##
			## Procent av totalen
			##
			
			$proc = ($proc >= 10) ? "$proc\%" : "<font color=\"silver\">$proc%</font>";
			
			print HTML "<td>$proc</td>";
			
			##
			## Delta
			##
			
			$value = my $delta = &shorten_num( $stat->{"delta_$_"} );
			
			if ($value > 0)
			{
				$value .= 'pps';
				$value = "<b>$value</b>" if ($value =~ /(kpps|Mbps)$/);
				$value = "<font color=\"silver\">$value</font>" if ($delta < 5);
			}
			else
			{
				$value = '';
			}
			
			print HTML "<td>$value</td>";
			print HTML "<td></td>" unless ($_ eq 'rx1024ToMa');
		}
		
		print HTML "</tr>\n";
	}
	
	my $created = localtime;
	my $tmp = $runtime;
	$tmp =~ s/(\.\d\d).+$/$1/;
	
	print HTML <<END;
</table>

<p>
<div style="font-family: Verdana, Geneva, Arial, Helvetica, sans-serif; font-size: 9;">
$created - loop $loop - The speeds are averages since last loop, around $refresh_rate seconds (loop took $tmp secs)<br>
<a href="http://$SWITCH" target="delladm">$system{'productName'}</a>, name $system{'systemName'}, location $system{'locationName'}, asset $system{'assetTag'}, mac $system{'macAddr'}, ver $system{'fwVer'}/$system{'bootVer'}
</div>

</center>

</body>
</html>
END
	
	close HTML;
}

##
## Logga in mot Dell-switchen fr att f sessions-cookien
##

sub login
{
	##
	## Fetched the loginpage to get the initial Session-value
	##
	
	print "Logging in info $SWITCH...\n";
	
	my $res = $ua->get("http://$SWITCH/login11.htm");
	die "Unable to fetch /login11.htm - ", $res->status_line, "\n" unless ($res->is_success);
	
	my $data = $res->content;
	my $session = '';
	
	# Finds the Session-value, needed to compute the password that is sent when logging in
	if ($data =~ /^<input type="hidden" name="Session" value="(.*)">/m)
	{
		$session = $1;
	}
	
	unless ($session)
	{
		die "Unable to find variable Session in the page! $data";
	}
	
	if ($session =~ /^0+$/)
	{
		die "Session $session is invalid! You may need to wait a while, or reboot the switch";
	}
	
	print "Found 'Session': $session\n";
	#$session = '8c6e05969fb0748f7fa239628432ffe4';
	
	##
	## Loggar in och fr frhoppningsvis en cookie
	## Mste kra med 
	##
	
	my $md5_password = md5_hex($USERNAME . $PASSWORD . $session);
	print "Computed MD5: $md5_password\n";
	
	my $sock = Net::HTTP->new(Host => $SWITCH) || die $@;
	$sock->write_request(POST => "/tgi/login.tgi",
					'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
					'Content-Type' => 'application/x-www-form-urlencoded',
					'Cache-Control' => 'no-cache',
					'Accept' => '*/*',
					"Username=$USERNAME&Password=$md5_password&Session=$session"
			);
	
	my($code, $mess, %h) = $sock->read_response_headers;
	
	unless ($code == 302)
	{
		die "Didn't recieve code 302 in return! $mess";
	}
	
	##
	## Finding the Cookie we should recieve when logged in successfully
	##
	## Set-Cookie : SSID=0f9a03b3d15acd65af5dfcf6550c5eb6; path=/
	##
	
	unless ($h{'Set-Cookie'})
	{
		foreach (sort keys %h)
		{
			print "$_ = $h{$_}\n";
		}
			
		die "Didn't recieve cookie! $h{'Set-Cookie'}";
	}
	
	$h{'Set-Cookie'} =~ /(SSID=.+?)\;/ || die "Unable to interpret SSID from $h{'Set-Cookie'}!";
	$cookie = $1;
	
	print "Logged in, using session-cookie: $cookie\n";
	
	##
	## Caching the session for future runs
	## Prevents hanging of the web-interface if script is restarted un too often
	##
	
	if ($COOKIE_FILE)
	{
		open (COOKIE, ">$COOKIE_FILE") || die "Unable to save the cookie to file $COOKIE_FILE!";
		print COOKIE $cookie;
		close COOKIE;
	}
	else
	{
		print "Doesn't cache session\n";
	}
}

##
## Resets stats for a port
##

sub clear_stat
{
	my $portnum = shift;
	
	my $sock = Net::HTTP->new(Host => $SWITCH) || die $@;
	$sock->write_request(POST => "/tgi/portstats.tgi",
			'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
			'Content-Type' => 'application/x-www-form-urlencoded',
			'Cache-Control' => 'no-cache',
			'Accept' => '*/*',
			'Cookie' => $cookie,
			'Referer' => "http://$SWITCH/portStats.htm?PortNo=$portnum"
,"PortNo=$portnum&PortNo\%24select=$portnum&RefreshRate\%24select=15000&clrCnts=OK"
	);
	
	my ($code, $mess, %h) = $sock->read_response_headers;
	#print "$code : $mess\n";
	
	#foreach (sort keys %h)
	#{
	#	print "  $_ : $h{$_}\n";
	#}
	
	#unless ($code == 302)
	#{
	#	print "Fick inte tillbaka code 302! $mess\n";
	#}
	
	#my $buf;
	#$sock->read_entity_body($buf, 1024);
	#print $buf;
	
	return ($code == 302);		# 302 = Probably OK
}

##
## Reading systeminfo that doesn't change
##

sub get_system_info
{
	print "Polling system info...\n";
	
	my $res = $ua->get("http://$SWITCH/system.htm",
	       'Cookie' => $cookie
		 );
	
	die "Unable to fetch /system.htm - ", $res->status_line, "\n" unless ($res->is_success);
	
	# Parsing content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($data =~ /^<title>Login<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		print "Session invalid\n";
		
		$cookie = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	# var productName="PowerConnect 2716";
	# var macAddr="00:13:72:F5:68:68";
	# var assetTag="";
	# var serviceTag="CCZR291";
	# var systemName="";
	# var locationName="";
	# var sysUpTime="0005597773";
	# var serialNum="CN0W88752829863M0026";
	# var fwVer="1.0.1.07";
	# var bootVer="1008";
	
	my $row = 0;
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		last if (/^<\/script>$/);	# May need adjustment for future firmwares
		next unless (/^var (.+)="(.*)";$/);
		
		#next if ($stat{$name});		# Skips already found variables, a bug in the current firmware
		
		#print "  $1 -> $2\n";
		$system{$1} = $2;
	}
	
	return 1;
}


##
## Rewrites 1024 as 1k, with a given number of decimals
##

sub shorten_num
{
	my $byte = shift;
	my $digits = shift || 3;
	my $unit = '';
	
	if ($byte > 1024)
	{
		$unit = 'k';
		$byte /= 1024;
	}
	if ($byte > 1024)
	{
		$unit = 'M';
		$byte /= 1024;
	}
	if ($byte > 1024)
	{
		$unit = 'G';
		$byte /= 1024;
	}
	if ($byte > 1024)
	{
		$unit = 'T';
		$byte /= 1024;
	}
	
	$byte = ($byte >= 1000) ? substr($byte, 0, 4) : substr($byte, 0, $digits);
	$byte =~ s/(\.\d*)0+$/$1/;
	$byte =~ s/\.$//;
	
	return $byte . $unit;
}

